#ifndef TRANSPORT_CLIENT_H
#define TRANSPORT_CLIENT_H

#include <time.h>
#include <opensrf/transport_session.h>
#include <opensrf/utils.h>
#include <opensrf/log.h>

#ifdef __cplusplus
extern "C" {
#endif

struct message_list_struct;

// ---------------------------------------------------------------------------
// Our client struct.  We manage a list of messages and a controlling session
// ---------------------------------------------------------------------------
struct transport_client_struct {
	transport_message* msg_q_head;
	transport_message* msg_q_tail;
	transport_session* session;
	int error;
};
typedef struct transport_client_struct transport_client;

// ---------------------------------------------------------------------------
// Allocates and initializes a transport_client.  This does no connecting
// The user must call client_free(client) when finished with the allocated
// object.
// if port > 0 => connect via TCP
// else if unix_path != NULL => connect via UNIX socket
// ---------------------------------------------------------------------------
transport_client* client_init( const char* server, int port, const char* unix_path, int component );


// ---------------------------------------------------------------------------
// Connects to the Jabber server with the provided information. Returns 1 on
// success, 0 otherwise.
// ---------------------------------------------------------------------------
int client_connect( transport_client* client, 
		const char* username, const char* password, const char* resource,
		int connect_timeout, enum TRANSPORT_AUTH_TYPE auth_type );


int client_disconnect( transport_client* client );

// ---------------------------------------------------------------------------
// De-allocates memory associated with a transport_client object.  Users
// must use this method when finished with a client object.
// ---------------------------------------------------------------------------
int client_free( transport_client* client );

// ---------------------------------------------------------------------------
// Sends the given message.  The message must at least have the recipient
// field set.
// ---------------------------------------------------------------------------
int client_send_message( transport_client* client, transport_message* msg );

// ---------------------------------------------------------------------------
// Returns 1 if this client is currently connected to the server, 0 otherwise
// ---------------------------------------------------------------------------
int client_connected( const transport_client* client );

// ---------------------------------------------------------------------------
// If there are any messages in the message list, the 'oldest' message is
// returned.  If not, this function will wait at most 'timeout' seconds 
// for a message to arrive.  Specifying -1 means that this function will not
// return unless a message arrives.
// ---------------------------------------------------------------------------
transport_message* client_recv( transport_client* client, int timeout );

#ifdef __cplusplus
}
#endif

#endif
